/* Gamepad3.C
   2/20/02
   REV. A0
   Use with Rev. P1 Board
   Paul Hackmann
   CD-i Gamepad Converter Cable Code
   Compiled with CCS Pic C Compiler
*/

   #INCLUDE "d:\picc2\examples\16C715.H"
   #FUSES   XT,PUT,NOPROTECT,NOWDT  //No code protect, no watchdog, powerup timer
   #ZERO_RAM
   #USE delay(clock=4000000) //set delay speed for 4mhz clock
   
   #USE STANDARD_IO(A)		
   #USE STANDARD_IO(B)

   #byte porta = 5
   #byte portb = 6

   unsigned int xcenter;
   unsigned int ycenter;
   
//code

unsigned int ReadAD(int channel)      //read the pin voltage via A/D
{
   unsigned long int Value1, Value2, Value3;
   unsigned int Value;

   set_adc_channel(channel);     //Set input to certain pin

   delay_us(50);          //wait for input to stabilize, important for accuracy

   Value1 = Read_ADC();        //read data from pin once

   delay_us(50);          //wait for A/D to refresh

   Value2 = Read_ADC();       //read data from pin twice

   delay_us(50);          //wait for A/D to refresh

   Value3 = Read_ADC();       //read data from pin a third time
   
   if((Value1 + Value2 + Value3) != 0)
      Value = (Value1 + Value2 + Value3) / 3;
   else
      Value = 0;

   return Value;
}

#SEPARATE
void Calibration()
{
   long int tempx,tempy;

   xcenter = ReadAD(0);
   ycenter = ReadAD(1);

   tempx = (((long int)25 * xcenter) / 7) - 627;
   tempy = (((long int)25 * ycenter) / 7) - 627;

   xcenter = tempx;
   ycenter = tempy;

}

void main()
{
   int padbyte0, padbyte1, padbyte2, oldpadbyte0, oldpadbyte1, oldpadbyte2;
   unsigned int x,y,oldx,oldy,speed;
   long int tempx, tempy;
   unsigned int speedadj;

   setup_adc_ports(ALL_ANALOG);     //initialize A/D ports for analog use
   setup_adc(ADC_CLOCK_DIV_8);      //set the A/D clock speed
   
   oldx = 210;
   oldy = 210;
   speed = 255;

   output_bit(PIN_B7,1);  //verifies the unit is running
   delay_ms(5);
   output_bit(PIN_B7,0);
   delay_ms(5);
   output_bit(PIN_B7,1);
   delay_ms(5);
   output_bit(PIN_B7,0);

   Calibration();	//finds the center of the directional controls

/*************************************** DEBUG ******************************************/
//   #USE RS232(baud=9600, xmit=PIN_B7, rcv=PIN_B6)
//   printf("3. %u ",xcenter);
//   printf("4. %u ",ycenter);
/*************************************** DEBUG ******************************************/

   #USE RS232(baud=1200, xmit=PIN_B5, rcv=PIN_B6, invert)
   while(TRUE)  //loop forever
   {
     oldpadbyte0 = 0;
     oldpadbyte1 = 0;
     oldpadbyte2 = 0;
      
     do{		    //wait for cd-i player to assert the rts line
     }while(input(PIN_B4) != 0);
     
     delay_us(500);  //wait a smidge for the cd-i player to be ready to receive
     
     putc(0b11001010);  //send the device id

     while(input(PIN_B4) == 0)  //while rts asserted
     {
       padbyte0 = 0b11000000;  //initialize data bytes
       padbyte1 = 0b10000000;
       padbyte2 = 0b10000000;

       speed = ReadAD(2);	//read in speed adjustment voltage
       speed = speed/2;

       x = ReadAD(0);	//read voltage at pin A0
       y = ReadAD(1);	//read voltage at pin A1

/*************************************** DEBUG ******************************************/
//       #USE RS232(baud=9600, xmit=PIN_B7, rcv=PIN_B6)
//       printf("5. %u ",speed);
//       printf("6. %u ",x);
//       printf("7. %u ",y);
/*************************************** DEBUG ******************************************/

       if(x < 176)	//if it is too low, adjust it up
         x = 176;
       if(y < 176)
         y = 176;

       x = ((long int)x + oldx) / 2;	//average the previous value with the current one
       y = ((long int)y + oldy) / 2;

/*************************************** DEBUG ******************************************/
//       printf("8. %u ",x);
//       printf("9. %u ",y);
//       printf("10. %u ",oldx);
//       printf("11. %u ",oldy);
/*************************************** DEBUG ******************************************/

       oldx = x;			//save the current value
       oldy = y;

       tempx = (((long int)25 * x) / 7) - 627;	//convert the value to a 0 to 255 value
       tempy = (((long int)25 * y) / 7) - 627;

       if(tempx >= 255)		//ceiling it at 255
         x = 255;
       else
         x = tempx;

       if(tempy >= 255)
         y = 255;
       else
         y = tempy;       
       
       speedadj = ((signed long int)100 * speed) / 127;		//speed percentage

/*************************************** DEBUG ******************************************/
//       printf("12. %u ",x);
//       printf("13. %u ",y);
//       printf("14. %u ",speedadj);
//       #USE RS232(baud=1200, xmit=PIN_B5, rcv=PIN_B6, invert)
/*************************************** DEBUG ******************************************/

       if((x < (xcenter - 18)) && (x > 0))		//convert right direction
       {
	 if((unsigned int)(((signed long int)100 * (xcenter - x)) / xcenter) > speedadj)  //percentage of x to xcenter greater than speed percentage?
	   x = 127 - speed;				//set to max speed
	 else
	   x = 127 - (unsigned int)((signed long int)127 * (xcenter - x) / xcenter);

/*************************************** DEBUG ******************************************/
//         #USE RS232(baud=9600, xmit=PIN_B7, rcv=PIN_B6)
//         printf("15. %u ",x);
//         #USE RS232(baud=1200, xmit=PIN_B5, rcv=PIN_B6, invert)
/*************************************** DEBUG ******************************************/

         x = x ^ 0b01111111;		//invert
         x = x + 1;			//add 1
         padbyte1 = padbyte1 | x;
         padbyte1 = padbyte1 & 0b10111111;
         if((x & 0b01000000) != 0)
	   padbyte0 = padbyte0 | 0b00000001;
       }

       else if(x > (xcenter + 18))			//convert left direction
       {
	 if((unsigned int)(((signed long int)100 * (x - xcenter)) / (255 - xcenter)) > speedadj)
	   x = 127 + speed;
	 else
	   x = 127 + (((signed long int)127 * (x - xcenter)) / (255 - xcenter));

/*************************************** DEBUG ******************************************/
//         #USE RS232(baud=9600, xmit=PIN_B7, rcv=PIN_B6)
//         printf("16. %u ",x);
//         #USE RS232(baud=1200, xmit=PIN_B5, rcv=PIN_B6, invert)
/*************************************** DEBUG ******************************************/

         x = x ^ 0b01111111;		//invert
         x = x + 1;			//add 1
         padbyte1 = padbyte1 | x;
         padbyte1 = padbyte1 & 0b10111111;
         if((x & 0b01000000) != 0)
	   padbyte0 = padbyte0 | 0b00000011;
         else
           padbyte0 = padbyte0 | 0b00000010; 
       }

       if((y < (ycenter - 18)) && (y > 0))		//convert down direction
       {
	 if((unsigned int)(((signed long int)100 * (ycenter - y)) / ycenter) > speedadj)  //percentage of y to ycenter greater than speed percentage?
	   y = 127 - speed;				//set to max speed
	 else
	   y = 127 - (unsigned int)((signed long int)127 * (ycenter - y) / ycenter);

/*************************************** DEBUG ******************************************/
//         #USE RS232(baud=9600, xmit=PIN_B7, rcv=PIN_B6)
//         printf("17. %u ",y);
//         #USE RS232(baud=1200, xmit=PIN_B5, rcv=PIN_B6, invert)
/*************************************** DEBUG ******************************************/

	 y = y ^ 0b01111111;		//invert
         y = y + 1;			//add 1
         padbyte2 = padbyte2 | y;
         padbyte2 = padbyte2 & 0b10111111;
         if((y & 0b01000000) != 0)
	   padbyte0 = padbyte0 | 0b00000100;
       }

       else if(y > (ycenter + 18))			//convert up direction
       {
	 if((unsigned int)(((signed long int)100 * (y - ycenter)) / (255 - ycenter)) > speedadj)
	   y = 127 + speed;
	 else
	   y = 127 + (((signed long int)127 * (y - ycenter)) / (255 - ycenter));

/*************************************** DEBUG ******************************************/
//         #USE RS232(baud=9600, xmit=PIN_B7, rcv=PIN_B6)
//         printf("18. %u ",y);
//         #USE RS232(baud=1200, xmit=PIN_B5, rcv=PIN_B6, invert)
/*************************************** DEBUG ******************************************/

         y = y ^ 0b01111111;		//invert
	 y = y + 1;			//add 1
         padbyte2 = padbyte2 | y;
         padbyte2 = padbyte2 & 0b10111111;
         if((y & 0b01000000) != 0)
	   padbyte0 = padbyte0 | 0b00001100;
         else
           padbyte0 = padbyte0 | 0b00001000; 
       }

/*************************************** DEBUG ******************************************/
//         #USE RS232(baud=9600, xmit=PIN_B7, rcv=PIN_B6)
//         printf("\n",y);
//         #USE RS232(baud=1200, xmit=PIN_B5, rcv=PIN_B6, invert)
/*************************************** DEBUG ******************************************/

       if(input(PIN_B1) != 1)
       {
      	 padbyte0 = padbyte0 | 0b00110000;  //button 4 on      
       }
         else if(input(PIN_B2) != 1)
         {
      	   padbyte0 = padbyte0 | 0b00110000;  //button 3 on
      	 }
      	
       if(input(PIN_B0) != 1)
       	 padbyte0 = padbyte0 | 0b00010000;  //button 2 on
      	
       if(input(PIN_B3) != 1)
      	 padbyte0 = padbyte0 | 0b00100000;  //button 1 on
      	      
       if((padbyte0 != oldpadbyte0) || (padbyte1 != 0b10000000) || (padbyte2 != 0b10000000) || ((padbyte0 & 0b00001111) != 0))	//see if button states have changed
       {     
         if(input(PIN_B4) == 0)
         {
           putc(padbyte0);	//send byte 0
           if(input(PIN_B4) == 0)
           {
             putc(padbyte1);	//send byte 1
             if(input(PIN_B4) == 0)
             {
               putc(padbyte2);	//send byte 2
             }
           }
         }
       }
      
       oldpadbyte0 = padbyte0;	//save button states to oldpadbyteX
       oldpadbyte1 = padbyte1;
       oldpadbyte2 = padbyte2;
     }
   }
}


